@@ -52,7 +52,7 @@ der_coder(const void *buf, size_t size, void *app_key)
 }
 
 /*
- * FIXME (next iteration) This isn't efficient, consider implement DER decoding
+ * TODO (performance) This isn't efficient, consider implement DER decoding
  * or something better.
  */
 static int
@@ -96,9 +96,6 @@ asn1_decode(const void *buffer, size_t buffer_size,
 	rval = ber_decode(&s_codec_ctx, descriptor, result, buffer,
 	    buffer_size);
 	if (rval.code != RC_OK) {
-		if (rval.code == RC_FAIL && errno == ENOMEM)
-			pr_enomem();
-
 		/* Must free partial object according to API contracts. */
 		ASN_STRUCT_FREE(*descriptor, *result);
 		/* We expect the data to be complete; RC_WMORE is an error. */
