@@ -38,6 +38,15 @@
 
 #define ARRAY_LEN(array) (sizeof(array) / sizeof((array)[0]))
 
+void panic_on_fail(int, char const *);
+
+/*
+ * Mutex wrappers. They are just a bunch of boilerplate, and removal of
+ * unrecoverable resulting error codes.
+ */
+void mutex_lock(pthread_mutex_t *);
+void mutex_unlock(pthread_mutex_t *);
+
 /*
  * rwlock wrappers. They are just a bunch of boilerplate, and removal of
  * unrecoverable resulting error codes.
@@ -50,7 +59,7 @@ typedef int (*process_file_cb)(char const *, void *);
 int process_file_or_dir(char const *, char const *, bool, process_file_cb,
     void *);
 
-typedef int (*pr_errno_cb)(int, const char *, ...);
+typedef int (*pr_errno_cb)(const char *, ...);
 bool valid_file_or_dir(char const *, bool, bool, pr_errno_cb);
 
 int create_dir_recursive(char const *);
