@@ -1,6 +1,9 @@
 #include "log.h"
 
+#ifdef BACKTRACE_ENABLED
 #include <execinfo.h>
+#endif
+
 #include <openssl/bio.h>
 #include <openssl/err.h>
 #include <pthread.h>
@@ -66,6 +69,7 @@ static pthread_mutex_t logck;
 void
 print_stack_trace(char const *title)
 {
+#ifdef BACKTRACE_ENABLED
 #define STACK_SIZE 64
 
 	void *array[STACK_SIZE];
@@ -97,6 +101,7 @@ print_stack_trace(char const *title)
 	}
 
 	free(strings);
+#endif /* BACKTRACE_ENABLED */
 }
 
 static void init_config(struct log_config *cfg, bool unit_tests)
@@ -109,6 +114,8 @@ static void init_config(struct log_config *cfg, bool unit_tests)
 	cfg->facility = LOG_DAEMON;
 }
 
+#ifdef BACKTRACE_ENABLED
+
 static void
 sigsegv_handler(int signum)
 {
@@ -135,6 +142,8 @@ sigsegv_handler(int signum)
 	kill(getpid(), signum);
 }
 
+#endif
+
 /*
  * Register better handlers for some signals.
  *
@@ -143,6 +152,7 @@ sigsegv_handler(int signum)
 static void
 register_signal_handlers(void)
 {
+#ifdef BACKTRACE_ENABLED
 	struct sigaction action;
 	void* dummy;
 
@@ -163,8 +173,10 @@ register_signal_handlers(void)
 		 * Not fatal; it just means we will not print stack traces on
 		 * Segmentation Faults.
 		 */
-		pr_op_errno(errno, "SIGSEGV handler registration failure");
+		pr_op_err("SIGSEGV handler registration failure: %s",
+		    strerror(errno));
 	}
+#endif
 
 	/*
 	 * SIGPIPE can be triggered by any I/O function. libcurl is particularly
@@ -190,7 +202,8 @@ register_signal_handlers(void)
 		 * will die silently.
 		 * But they're somewhat rare, so whatever.
 		 */
-		pr_op_errno(errno, "SIGPIPE handler registration failure");
+		pr_op_err("SIGPIPE handler registration failure: %s",
+		    strerror(errno));
 	}
 }
 
@@ -220,11 +233,11 @@ log_setup(bool unit_tests)
 	error = pthread_mutex_init(&logck, NULL);
 	if (error) {
 		fprintf(ERR.stream, "pthread_mutex_init() returned %d: %s\n",
-		    error, strerror(abs(error)));
+		    error, strerror(error));
 		if (!unit_tests)
 			syslog(LOG_ERR | op_config.facility,
 			    "pthread_mutex_init() returned %d: %s",
-			    error, strerror(abs(error)));
+			    error, strerror(error));
 		return error;
 	}
 
@@ -504,13 +517,20 @@ pr_op_warn(const char *format, ...)
 }
 
 int
-__pr_op_err(int error, const char *format, ...)
+pr_op_err(const char *format, ...)
+{
+	PR_SIMPLE(LOG_ERR, op_config);
+	return -EINVAL;
+}
+
+int
+pr_op_err_st(const char *format, ...)
 {
 	PR_SIMPLE(LOG_ERR, op_config);
 	lock_mutex();
 	print_stack_trace(NULL);
 	unlock_mutex();
-	return error;
+	return -EINVAL;
 }
 
 void
@@ -537,40 +557,40 @@ pr_val_warn(const char *format, ...)
 }
 
 int
-__pr_val_err(int error, const char *format, ...)
+pr_val_err(const char *format, ...)
 {
 	PR_SIMPLE(LOG_ERR, val_config);
-	return error;
+	return -EINVAL;
 }
 
 struct crypto_cb_arg {
 	unsigned int stack_size;
-	int (*error_fn)(int, const char *, ...);
+	int (*error_fn)(const char *, ...);
 };
 
 static int
 log_crypto_error(const char *str, size_t len, void *_arg)
 {
 	struct crypto_cb_arg *arg = _arg;
-	arg->error_fn(0, "-> %s", str);
+	arg->error_fn("-> %s", str);
 	arg->stack_size++;
 	return 1;
 }
 
 static int
-crypto_err(struct log_config *cfg, int (*error_fn)(int, const char *, ...))
+crypto_err(struct log_config *cfg, int (*error_fn)(const char *, ...))
 {
 	struct crypto_cb_arg arg;
 
-	error_fn(0, "libcrypto error stack:");
+	error_fn("libcrypto error stack:");
 
 	arg.stack_size = 0;
 	arg.error_fn = error_fn;
 	ERR_print_errors_cb(log_crypto_error, &arg);
 	if (arg.stack_size == 0)
-		error_fn(0, "   <Empty>");
+		error_fn("   <Empty>");
 	else
-		error_fn(0, "End of libcrypto stack.");
+		error_fn("End of libcrypto stack.");
 
 	return -EINVAL;
 }
@@ -591,7 +611,7 @@ int
 op_crypto_err(const char *format, ...)
 {
 	PR_SIMPLE(LOG_ERR, op_config);
-	return crypto_err(&op_config, __pr_op_err);
+	return crypto_err(&op_config, pr_op_err);
 }
 
 /**
@@ -610,7 +630,7 @@ int
 val_crypto_err(const char *format, ...)
 {
 	PR_SIMPLE(LOG_ERR, val_config);
-	return crypto_err(&val_config, __pr_val_err);
+	return crypto_err(&val_config, pr_val_err);
 }
 
 int
