@@ -5,6 +5,12 @@
 #include <syslog.h>
 #include <time.h>
 #include <openssl/asn1.h>
+#include <openssl/opensslv.h>
+
+#if OPENSSL_VERSION_MAJOR >= 3
+#include <openssl/core_names.h>
+#endif
+
 #include <sys/socket.h>
 
 #include "algorithm.h"
@@ -185,126 +191,11 @@ spki_cmp(X509_PUBKEY *tal_spki, X509_PUBKEY *cert_spki,
 	return 0;
 }
 
-/** Call whenever the public key is different and the subject is the same**/
-static int
-cert_different_pk_err(void)
-{
-	return 1;
-}
-
 /*
- * Beware: this function skips a lot (maybe all) RPKI validations for a signed
- * object, and clearly expects that the signed object has at least one
- * certificate.
- *
- * Allocates @result, don't forget to release it.
- *
- * It has been placed here to minimize the risk of misuse.
+ * https://mailarchive.ietf.org/arch/msg/sidrops/mXWbCwh6RO8pAtt7N30Q9m6jUws/
+ * Concensus (in mailing list as well as Discord) seems to be "do not check
+ * subject name uniqueness."
  */
-static int
-cert_from_signed_object(struct rpki_uri *uri, uint8_t **result, int *size)
-{
-	struct signed_object sobj;
-	ANY_t *cert;
-	unsigned char *tmp;
-	int error;
-
-	error = signed_object_decode(&sobj, uri);
-	if (error)
-		return error;
-
-	cert = sobj.sdata.decoded->certificates->list.array[0];
-
-	tmp = malloc(cert->size);
-	if (tmp == NULL) {
-		signed_object_cleanup(&sobj);
-		return pr_enomem();
-	}
-
-	memcpy(tmp, cert->buf, cert->size);
-
-	*result = tmp;
-	(*size) = cert->size;
-
-	signed_object_cleanup(&sobj);
-	return 0;
-}
-
-static int
-check_dup_public_key(bool *duplicated, char const *file, void *arg)
-{
-	X509 *curr_cert = arg; /* Current cert */
-	X509 *rcvd_cert;
-	X509_PUBKEY *curr_pk, *rcvd_pk;
-	struct rpki_uri *uri;
-	uint8_t *tmp;
-	int tmp_size;
-	int error;
-
-	uri = NULL;
-	rcvd_cert = NULL;
-	tmp_size = 0;
-
-	error = uri_create_rsync_str(&uri, file, strlen(file));
-	if (error)
-		return error;
-
-	/* Check if it's '.cer', otherwise treat as a signed object */
-	if (uri_is_certificate(uri)) {
-		error = certificate_load(uri, &rcvd_cert);
-		if (error)
-			goto free_uri;
-	} else {
-		error = cert_from_signed_object(uri, &tmp, &tmp_size);
-		if (error)
-			goto free_uri;
-
-		rcvd_cert = d2i_X509(NULL, (const unsigned char **) &tmp,
-		    tmp_size);
-		free(tmp); /* Release at once */
-		if (rcvd_cert == NULL) {
-			error = val_crypto_err("Signed object's '%s' 'certificate' element does not decode into a Certificate",
-			    uri_val_get_printable(uri));
-			goto free_uri;
-		}
-	}
-
-	curr_pk = X509_get_X509_PUBKEY(curr_cert);
-	if (curr_pk == NULL) {
-		error = val_crypto_err("X509_get_X509_PUBKEY() returned NULL");
-		goto free_cert;
-	}
-
-	rcvd_pk = X509_get_X509_PUBKEY(rcvd_cert);
-	if (rcvd_pk == NULL) {
-		error = val_crypto_err("X509_get_X509_PUBKEY() returned NULL");
-		goto free_cert;
-	}
-
-	/*
-	 * The function response will be:
-	 *   < 0 if there's an error
-	 *   = 0 if the PKs are equal
-	 *   > 0 if the PKs are different
-	 */
-	error = spki_cmp(curr_pk, rcvd_pk, cert_different_pk_err,
-	    cert_different_pk_err);
-
-	if (error < 0)
-		goto free_cert;
-
-	/* No error, a positive value means the name is duplicated */
-	if (error)
-		(*duplicated) = true;
-	error = 0;
-
-free_cert:
-	X509_free(rcvd_cert);
-free_uri:
-	uri_refput(uri);
-	return error;
-}
-
 static int
 validate_subject(X509 *cert)
 {
@@ -321,9 +212,6 @@ validate_subject(X509 *cert)
 		return error;
 	pr_val_debug("Subject: %s", x509_name_commonName(name));
 
-	error = x509stack_store_subject(validation_certstack(state), name,
-	    check_dup_public_key, cert);
-
 	x509_name_put(name);
 	return error;
 }
@@ -409,6 +297,87 @@ fail1:
 static int
 validate_subject_public_key(X509_PUBKEY *pubkey)
 {
+#if OPENSSL_VERSION_MAJOR >= 3
+
+	const size_t EXPECTED_BITS = 2048;
+	const size_t EXPECTED_EXPONENT = 65537;
+
+	EVP_PKEY *key;
+	int key_type;
+	size_t bits;
+	size_t exponent;
+
+	key = X509_PUBKEY_get0(pubkey);
+	if (key == NULL)
+		return val_crypto_err("X509_PUBKEY_get0() returned NULL");
+
+	key_type = EVP_PKEY_get_base_id(key);
+	if (key_type != EVP_PKEY_RSA && key_type != EVP_PKEY_RSA2)
+		return val_crypto_err("The public key type is not RSA: %u",
+		    key_type);
+
+	/*
+	 * man 7 EVP_PKEY-RSA:
+	 *
+	 * > "bits" (OSSL_PKEY_PARAM_RSA_BITS) <unsigned integer>
+	 * > The value should be the cryptographic length for the RSA
+	 * > cryptosystem, in bits.
+	 * > "primes" (OSSL_PKEY_PARAM_RSA_PRIMES) <unsigned integer> (...)
+	 * > "e" (OSSL_PKEY_PARAM_RSA_E) <unsigned integer> (...)
+	 *
+	 * I'm having a hard time demonstrating the equality of "cryptographic
+	 * length" and "modulus length" using authoritative sources... but I
+	 * mean, it makes sense in context:
+	 *
+	 * Those three arguments from EVP_PKEY-RSA seem to be low-level
+	 * equivalents to the ones exposed on `man 1 openssl genpkey`:
+	 *
+	 * > rsa_keygen_bits:numbits
+	 * > The number of bits in the generated key. If not specified 2048 is
+	 * > used.
+	 * > rsa_keygen_primes:numprimes (...)
+	 * > rsa_keygen_pubexp:value (...)
+	 *
+	 * And https://en.wikipedia.org/wiki/RSA_(cryptosystem):
+	 *
+	 * > n is used as the modulus for both the public and private keys.
+	 * > Its length, usually expressed in bits, is the *key length*.
+	 *
+	 * So "cryptographic length" equals "key length," and "key length"
+	 * equals "modulus length."
+	 *
+	 * *Shrug*. I'm sorry; it's the best I got.
+	 */
+	if (!EVP_PKEY_get_size_t_param(key, OSSL_PKEY_PARAM_RSA_BITS, &bits))
+		return val_crypto_err("Cannot extract the modulus length from the public key");
+	if (bits < EXPECTED_BITS)
+		return pr_val_err("Certificate's subjectPublicKey (RSAPublicKey) modulus lengths %zu bits, not %zu bits",
+		    bits, EXPECTED_BITS);
+	/*
+	 * I'm going to be a bit lenient with this, because a small amount of
+	 * forward compatibility is bound to be invaluably better than nothing.
+	 * (Notice this one's a warning.)
+	 */
+	if (bits > EXPECTED_BITS)
+		pr_val_warn("Certificate's subjectPublicKey (RSAPublicKey) modulus lengths %zu bits, not %zu bits",
+		    bits, EXPECTED_BITS);
+
+	/*
+	 * man 7 EVP_PKEY-RSA:
+	 * "e" (OSSL_PKEY_PARAM_RSA_E) <unsigned integer>
+	 * The RSA "e" value. The value may be any odd number greater than or
+	 * equal to 65537. The default value is 65537.
+	 */
+	if (!EVP_PKEY_get_size_t_param(key, OSSL_PKEY_PARAM_RSA_E, &exponent))
+		return val_crypto_err("Cannot extract the exponent from the public key");
+	if (exponent != EXPECTED_EXPONENT)
+		return pr_val_err("Certificate's subjectPublicKey (RSAPublicKey) exponent is %zu, not %zu",
+		    exponent, EXPECTED_EXPONENT);
+
+	return 0;
+
+#else /* if OPENSSL_VERSION_MAJOR < 3 */
+
 #define MODULUS 2048
 #define EXPONENT "65537"
 	const RSA *rsa;
@@ -422,9 +391,12 @@ validate_subject_public_key(X509_PUBKEY *pubkey)
 		return val_crypto_err("EVP_PKEY_get0_RSA() returned NULL");
 
 	modulus = RSA_bits(rsa);
-	if (modulus != MODULUS)
+	if (modulus < MODULUS)
 		return pr_val_err("Certificate's subjectPublicKey (RSAPublicKey) modulus is %d bits, not %d bits.",
 		    modulus, MODULUS);
+	if (modulus > MODULUS)
+		pr_val_warn("Certificate's subjectPublicKey (RSAPublicKey) modulus lengths %d bits, not %d bits",
+		    modulus, MODULUS);
 
 	RSA_get0_key(rsa, NULL, &exp, NULL);
 	if (exp == NULL)
@@ -446,6 +418,8 @@ validate_subject_public_key(X509_PUBKEY *pubkey)
 	return 0;
 #undef EXPONENT
 #undef MODULUS
+
+#endif /* OPENSSL_VERSION_MAJOR */
 }
 
 static int
@@ -1843,6 +1817,9 @@ get_certificate_type(X509 *cert, bool is_ta, enum cert_type *result)
 		return 0;
 	}
 
+	if (X509_check_purpose(cert, -1, -1) <= 0)
+		goto err;
+
 	if (X509_check_ca(cert) == 1) {
 		*result = CA;
 		return 0;
@@ -1853,6 +1830,7 @@ get_certificate_type(X509 *cert, bool is_ta, enum cert_type *result)
 		return 0;
 	}
 
+err:
 	*result = EE; /* Shuts up nonsense gcc 8.3 warning */
 	return pr_val_err("Certificate is not TA, CA nor BGPsec. Ignoring...");
 }
@@ -1975,8 +1953,7 @@ certificate_validate_aia(struct rpki_uri *caIssuers, X509 *cert)
 static int
 verify_mft_loc(struct rpki_uri *mft_uri)
 {
-	if (!valid_file_or_dir(uri_get_local(mft_uri), true, false,
-	    __pr_val_err))
+	if (!valid_file_or_dir(uri_get_local(mft_uri), true, false, pr_val_err))
 		return -EINVAL; /* Error already logged */
 
 	return 0;
