@@ -266,14 +266,25 @@ handle_child_thread(char **args, int fds[2][2])
 static int
 create_pipes(int fds[2][2])
 {
-	if (pipe(fds[0]) == -1)
-		return -pr_op_errno(errno, "Piping rsync stderr");
+	int error;
+
+	if (pipe(fds[0]) == -1) {
+		error = errno;
+		pr_op_err_st("Piping rsync stderr: %s", strerror(error));
+		return -error;
+	}
+
 	if (pipe(fds[1]) == -1) {
+		error = errno;
+
 		/* Close pipe previously created */
 		close(fds[0][0]);
 		close(fds[0][1]);
-		return -pr_op_errno(errno, "Piping rsync stdout");
+
+		pr_op_err_st("Piping rsync stdout: %s", strerror(error));
+		return -error;
 	}
+
 	return 0;
 }
 
@@ -300,7 +311,7 @@ log_buffer(char const *buffer, ssize_t read, int type, bool log_operation)
 		}
 		if (type == 0) {
 			if (log_operation)
-				pr_op_err(PRE_RSYNC "%s", cur);
+				pr_op_err_st(PRE_RSYNC "%s", cur);
 			pr_val_err(PRE_RSYNC "%s", cur);
 		} else {
 			pr_val_info(PRE_RSYNC "%s", cur);
@@ -322,10 +333,13 @@ read_pipe(int fd_pipe[2][2], int type, bool log_operation)
 	while (1) {
 		count = read(fd_pipe[type][0], buffer, sizeof(buffer));
 		if (count == -1) {
-			if (errno == EINTR)
+			error = errno;
+			if (error == EINTR)
 				continue;
 			close(fd_pipe[type][0]); /* Close read end */
-			return -pr_val_errno(errno, "Reading rsync buffer");
+			pr_val_err("rsync buffer read error: %s",
+			    strerror(error));
+			return -error;
 		}
 		if (count == 0)
 			break;
@@ -386,6 +400,7 @@ do_rsync(struct rpki_uri *uri, bool is_ta, bool log_operation)
 	if (error)
 		return error;
 
+	pr_val_info("rsync: %s", uri_get_global(uri));
 	if (log_val_enabled(LOG_DEBUG)) {
 		pr_val_debug("Executing RSYNC:");
 		for (i = 0; i < args_len + 1; i++)
@@ -422,8 +437,9 @@ do_rsync(struct rpki_uri *uri, bool is_ta, bool log_operation)
 			handle_child_thread(args, fork_fds);
 		}
 		if (child_pid < 0) {
-			pr_op_errno(errno, "Couldn't fork to execute rsync");
 			error = errno;
+			pr_op_err_st("Couldn't fork to execute rsync: %s",
+			   strerror(error));
 			/* Close all ends from the created pipes */
 			close(fork_fds[0][0]);
 			close(fork_fds[1][0]);
@@ -441,7 +457,7 @@ do_rsync(struct rpki_uri *uri, bool is_ta, bool log_operation)
 		do {
 			if (error == -1) {
 				error = errno;
-				pr_op_err("The rsync sub-process returned error %d (%s)",
+				pr_op_err_st("The rsync sub-process returned error %d (%s)",
 				    error, strerror(error));
 				if (child_status > 0)
 					break;
@@ -480,23 +496,23 @@ do_rsync(struct rpki_uri *uri, bool is_ta, bool log_operation)
 	if (WIFSIGNALED(child_status)) {
 		switch (WTERMSIG(child_status)) {
 		case SIGINT:
-			pr_op_err("RSYNC was user-interrupted. Guess I'll interrupt myself too.");
+			pr_op_err_st("RSYNC was user-interrupted. Guess I'll interrupt myself too.");
 			break;
 		case SIGQUIT:
-			pr_op_err("RSYNC received a quit signal. Guess I'll quit as well.");
+			pr_op_err_st("RSYNC received a quit signal. Guess I'll quit as well.");
 			break;
 		case SIGKILL:
-			pr_op_err("Killed.");
+			pr_op_err_st("Killed.");
 			break;
 		default:
-			pr_op_err("The RSYNC was terminated by a signal [%d] I don't have a handler for. Dunno; guess I'll just die.",
+			pr_op_err_st("The RSYNC was terminated by a signal [%d] I don't have a handler for. Dunno; guess I'll just die.",
 			    WTERMSIG(child_status));
 			break;
 		}
 		return -EINTR; /* Meh? */
 	}
 
-	pr_op_err("The RSYNC command died in a way I don't have a handler for. Dunno; guess I'll die as well.");
+	pr_op_err_st("The RSYNC command died in a way I don't have a handler for. Dunno; guess I'll die as well.");
 	return -EINVAL;
 release_args:
 	/* The happy path also falls here */
