@@ -7,7 +7,7 @@
 #include "data_structure/uthash_nonfatal.h"
 
 struct hashable_roa {
-	struct vrp data;
+	const struct vrp data;
 	UT_hash_handle hh;
 };
 
@@ -57,7 +57,7 @@ db_table_destroy(struct db_table *table)
 }
 
 int
-db_table_foreach_roa(struct db_table *table, vrp_foreach_cb cb, void *arg)
+db_table_foreach_roa(struct db_table const *table, vrp_foreach_cb cb, void *arg)
 {
 	struct hashable_roa *node, *tmp;
 	int error;
@@ -72,8 +72,8 @@ db_table_foreach_roa(struct db_table *table, vrp_foreach_cb cb, void *arg)
 }
 
 int
-db_table_foreach_router_key(struct db_table *table, router_key_foreach_cb cb,
-    void *arg)
+db_table_foreach_router_key(struct db_table const *table,
+    router_key_foreach_cb cb, void *arg)
 {
 	struct hashable_key *node, *tmp;
 	int error;
@@ -87,32 +87,26 @@ db_table_foreach_router_key(struct db_table *table, router_key_foreach_cb cb,
 	return 0;
 }
 
-static struct hashable_roa *
-create_roa(uint32_t asn, uint8_t max_length)
-{
-	struct hashable_roa *roa;
-
-	roa = malloc(sizeof(struct hashable_roa));
-	if (roa == NULL)
-		return NULL;
-	/* Needed by uthash */
-	memset(roa, 0, sizeof(struct hashable_roa));
-
-	roa->data.asn = asn;
-	roa->data.max_prefix_length = max_length;
-
-	return roa;
-}
-
 static int
-add_roa(struct db_table *table, struct hashable_roa *new)
+add_roa(struct db_table *table, struct hashable_roa const *stack_new)
 {
+	struct hashable_roa *new;
 	struct hashable_roa *old;
+	int error;
+
+	new = malloc(sizeof(struct hashable_roa));
+	if (new == NULL)
+		return pr_enomem();
+	memcpy(new, stack_new, sizeof(*new));
 
 	errno = 0;
 	HASH_REPLACE(hh, table->roas, data, sizeof(new->data), new, old);
-	if (errno)
-		return -pr_val_errno(errno, "ROA couldn't be added to hash table");
+	error = errno;
+	if (error) {
+		pr_val_err("ROA couldn't be added to hash table: %s",
+		    strerror(error));
+		return -error;
+	}
 	if (old != NULL)
 		free(old);
 
@@ -123,11 +117,16 @@ static int
 add_router_key(struct db_table *table, struct hashable_key *new)
 {
 	struct hashable_key *old;
+	int error;
 
 	errno = 0;
 	HASH_REPLACE(hh, table->router_keys, data, sizeof(new->data), new, old);
-	if (errno)
-		return -pr_val_errno(errno, "Router Key couldn't be added to hash table");
+	error = errno;
+	if (error) {
+		pr_val_err("Router Key couldn't be added to hash table: %s",
+		    strerror(error));
+		return -error;
+	}
 	if (old != NULL)
 		free(old);
 
@@ -175,40 +174,30 @@ int
 rtrhandler_handle_roa_v4(struct db_table *table, uint32_t asn,
     struct ipv4_prefix const *prefix4, uint8_t max_length)
 {
-	struct hashable_roa *roa;
-	int error;
+	struct hashable_roa new = {
+		.data.asn = asn,
+		.data.prefix.v4 = prefix4->addr,
+		.data.prefix_length = prefix4->len,
+		.data.max_prefix_length = max_length,
+		.data.addr_fam = AF_INET,
+	};
 
-	roa = create_roa(asn, max_length);
-	if (roa == NULL)
-		return pr_enomem();
-	roa->data.prefix.v4 = prefix4->addr;
-	roa->data.prefix_length = prefix4->len;
-	roa->data.addr_fam = AF_INET;
-
-	error = add_roa(table, roa);
-	if (error)
-		free(roa);
-	return error;
+	return add_roa(table, &new);
 }
 
 int
 rtrhandler_handle_roa_v6(struct db_table *table, uint32_t asn,
     struct ipv6_prefix const *prefix6, uint8_t max_length)
 {
-	struct hashable_roa *roa;
-	int error;
+	struct hashable_roa new = {
+		.data.asn = asn,
+		.data.prefix.v6 = prefix6->addr,
+		.data.prefix_length = prefix6->len,
+		.data.max_prefix_length = max_length,
+		.data.addr_fam = AF_INET6,
+	};
 
-	roa = create_roa(asn, max_length);
-	if (roa == NULL)
-		return pr_enomem();
-	roa->data.prefix.v6 = prefix6->addr;
-	roa->data.prefix_length = prefix6->len;
-	roa->data.addr_fam = AF_INET6;
-
-	error = add_roa(table, roa);
-	if (error)
-		free(roa);
-	return error;
+	return add_roa(table, &new);
 }
 
 int
@@ -240,24 +229,66 @@ rtrhandler_handle_router_key(struct db_table *table,
  */
 static int
 add_roa_deltas(struct hashable_roa *roas1, struct hashable_roa *roas2,
-    struct deltas *deltas, int op)
+    struct deltas *deltas, int op, char r1type)
 {
 	struct hashable_roa *n1; /* A node from @roas1 */
 	struct hashable_roa *n2; /* A node from @roas2 */
+	struct hashable_roa *tmp;
+	unsigned int r;
+	unsigned int roa1_count;
 	int error;
 
-	for (n1 = roas1; n1 != NULL; n1 = n1->hh.next) {
+	r = 0;
+	roa1_count = HASH_COUNT(roas1);
+
+	HASH_ITER(hh, roas1, n1, tmp) {
 		HASH_FIND(hh, roas2, &n1->data, sizeof(n1->data), n2);
 		if (n2 == NULL) {
-			error = deltas_add_roa(deltas, &n1->data, op);
+			error = deltas_add_roa(deltas, &n1->data, op,
+			    r1type, r, roa1_count);
 			if (error)
 				return error;
 		}
+		r++;
 	}
 
 	return 0;
 }
 
+void
+find_bad_vrp(char const *prefix, struct db_table *table)
+{
+	struct hashable_roa *node;
+	struct hashable_roa *tmp;
+	struct vrp const *vrp;
+	unsigned int roa_counter;
+	unsigned int roa_count;
+	char buffer[INET6_ADDRSTRLEN];
+
+	if (table == NULL)
+		return;
+
+	roa_counter = 0;
+	roa_count = HASH_COUNT(table->roas);
+
+	HASH_ITER(hh, table->roas, node, tmp) {
+		vrp = &node->data;
+		if (vrp->addr_fam != AF_INET && vrp->addr_fam != AF_INET6) {
+			pr_crit("%s: VRP corrupted! [%u %s/%u-%u %u] %u/%u "
+			    "(Please report this output to https://github.com/NICMx/FORT-validator/issues/89)",
+			    prefix,
+			    vrp->asn,
+			    addr2str6(&vrp->prefix.v6, buffer),
+			    vrp->prefix_length,
+			    vrp->max_prefix_length,
+			    vrp->addr_fam,
+			    roa_counter,
+			    roa_count);
+		}
+		roa_counter++;
+	}
+}
+
 static int
 add_router_key_delta(struct deltas *deltas, struct hashable_key *key, int op)
 {
@@ -300,10 +331,12 @@ compute_deltas(struct db_table *old, struct db_table *new,
 	if (error)
 		return error;
 
-	error = add_roa_deltas(new->roas, old->roas, deltas, FLAG_ANNOUNCEMENT);
+	error = add_roa_deltas(new->roas, old->roas, deltas, FLAG_ANNOUNCEMENT,
+	    'n');
 	if (error)
 		goto fail;
-	error = add_roa_deltas(old->roas, new->roas, deltas, FLAG_WITHDRAWAL);
+	error = add_roa_deltas(old->roas, new->roas, deltas, FLAG_WITHDRAWAL,
+	    'o');
 	if (error)
 		goto fail;
 	error = add_router_key_deltas(new->router_keys, old->router_keys,
