@@ -2,7 +2,7 @@
 #define SRC_RTR_PDU_SENDER_H_
 
 #include "pdu.h"
-#include "object/router_key.h"
+#include "types/router_key.h"
 #include "rtr/db/vrps.h"
 
 int send_serial_notify_pdu(int, uint8_t, serial_t);
