@@ -44,26 +44,6 @@ enum poll_verdict {
 	PV_STOP,
 };
 
-static void
-panic_on_fail(int error, char const *function_name)
-{
-	if (error)
-		pr_crit("%s() returned error code %d. This is too critical for a graceful recovery; I must die now.",
-		    function_name, error);
-}
-
-static void
-lock_mutex(void)
-{
-	panic_on_fail(pthread_mutex_lock(&lock), "pthread_mutex_lock");
-}
-
-static void
-unlock_mutex(void)
-{
-	panic_on_fail(pthread_mutex_unlock(&lock), "pthread_mutex_unlock");
-}
-
 static void
 cleanup_server(struct rtr_server *server)
 {
@@ -215,7 +195,7 @@ set_nonblock(int fd)
 	flags = fcntl(fd, F_GETFL);
 	if (flags == -1) {
 		error = errno;
-		pr_op_errno(error, "fcntl() to get flags failed");
+		pr_op_err_st("fcntl() to get flags failed: %s", strerror(error));
 		return error;
 	}
 
@@ -223,7 +203,7 @@ set_nonblock(int fd)
 
 	if (fcntl(fd, F_SETFL, flags) == -1) {
 		error = errno;
-		pr_op_errno(error, "fcntl() to set flags failed");
+		pr_op_err_st("fcntl() to set flags failed: %s", strerror(error));
 		return error;
 	}
 
@@ -261,7 +241,7 @@ create_server_socket(char const *input_addr, char const *hostname,
 	for (addr = addrs; addr != NULL; addr = addr->ai_next) {
 		fd = socket(addr->ai_family, SOCK_STREAM, 0);
 		if (fd < 0) {
-			pr_op_errno(errno, "socket() failed");
+			pr_op_err("socket() failed: %s", strerror(errno));
 			continue;
 		}
 
@@ -277,7 +257,8 @@ create_server_socket(char const *input_addr, char const *hostname,
 		/* enable SO_REUSEADDR */
 		if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &reuse,
 		    sizeof(int)) < 0) {
-			pr_op_errno(errno, "setsockopt(SO_REUSEADDR) failed");
+			pr_op_err("setsockopt(SO_REUSEADDR) failed: %s",
+			    strerror(errno));
 			close(fd);
 			continue;
 		}
@@ -285,22 +266,24 @@ create_server_socket(char const *input_addr, char const *hostname,
 		/* enable SO_REUSEPORT */
 		if (setsockopt(fd, SOL_SOCKET, SO_REUSEPORT, &reuse,
 		    sizeof(int)) < 0) {
-			pr_op_errno(errno, "setsockopt(SO_REUSEPORT) failed");
+			pr_op_err("setsockopt(SO_REUSEPORT) failed: %s",
+			    strerror(errno));
 			close(fd);
 			continue;
 		}
 
 		if (bind(fd, addr->ai_addr, addr->ai_addrlen) < 0) {
-			pr_op_errno(errno, "bind() failed");
+			pr_op_err("bind() failed: %s", strerror(errno));
 			close(fd);
 			continue;
 		}
 
-		error = getsockname(fd, addr->ai_addr, &addr->ai_addrlen);
-		if (error) {
+		if (getsockname(fd, addr->ai_addr, &addr->ai_addrlen) != 0) {
+			error = errno;
 			close(fd);
 			freeaddrinfo(addrs);
-			return pr_op_errno(errno, "getsockname() failed");
+			pr_op_err("getsockname() failed: %s", strerror(error));
+			return error;
 		}
 
 		port = (unsigned char)(addr->ai_addr->sa_data[0]) << 8;
@@ -311,8 +294,10 @@ create_server_socket(char const *input_addr, char const *hostname,
 		freeaddrinfo(addrs);
 
 		if (listen(fd, config_get_server_queue()) != 0) {
+			error = errno;
 			close(fd);
-			return pr_op_errno(errno, "listen() failure");
+			pr_op_err("listen() failure: %s", strerror(error));
+			return error;
 		}
 
 		server.fd = fd;
@@ -438,11 +423,10 @@ handle_accept_result(int client_fd, int err)
 		goto retry;
 #endif
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wlogical-op"
-	if (err == EAGAIN || err == EWOULDBLOCK)
+	if (err == EAGAIN)
+		goto retry;
+	if (err == EWOULDBLOCK)
 		goto retry;
-#pragma GCC diagnostic pop
 
 	pr_op_info("Client connection attempt not accepted: %s. Quitting...",
 	    strerror(err));
@@ -497,6 +481,7 @@ read_until_block(int fd, struct client_request *request)
 {
 	ssize_t read_result;
 	size_t offset;
+	int error;
 
 	request->nread = 0;
 
@@ -504,10 +489,12 @@ read_until_block(int fd, struct client_request *request)
 		read_result = read(fd, &request->buffer[offset],
 		    REQUEST_BUFFER_LEN - offset);
 		if (read_result == -1) {
-			if (errno == EAGAIN || errno == EWOULDBLOCK)
+			error = errno;
+			if (error == EAGAIN || error == EWOULDBLOCK)
 				return true; /* Ok, we have the full packet. */
 
-			pr_op_errno(errno, "Client socket read interrupted");
+			pr_op_err("Client socket read interrupted: %s",
+			    strerror(error));
 			return false;
 		}
 
@@ -717,9 +704,9 @@ fddb_poll(void)
 		}
 	}
 
-	lock_mutex();
+	mutex_lock(&lock);
 	apply_pollfds(pollfds, nclients);
-	unlock_mutex();
+	mutex_unlock(&lock);
 	/* Fall through */
 
 success:
@@ -786,8 +773,10 @@ void rtr_stop(void)
 
 	stop_server_thread = true;
 	error = pthread_join(server_thread, NULL);
-	if (error)
-		pr_op_errno(error, "pthread_join() returned %d", error);
+	if (error) {
+		pr_op_err("pthread_join() returned error %d: %s", error,
+		    strerror(error));
+	}
 
 	thread_pool_destroy(request_handlers);
 
@@ -801,7 +790,7 @@ rtr_foreach_client(rtr_foreach_client_cb cb, void *arg)
 	unsigned int i;
 	int error = 0;
 
-	lock_mutex();
+	mutex_lock(&lock);
 
 	ARRAYLIST_FOREACH(&clients, client, i) {
 		if (client->fd != -1) {
@@ -811,7 +800,7 @@ rtr_foreach_client(rtr_foreach_client_cb cb, void *arg)
 		}
 	}
 
-	unlock_mutex();
+	mutex_unlock(&lock);
 
 	return error;
 }
