@@ -4,6 +4,7 @@
 #include <pthread.h>
 #include <stdlib.h>
 #include <string.h>
+#include "common.h"
 #include "log.h"
 
 /*
@@ -79,26 +80,6 @@ struct thread_pool {
 	unsigned int thread_ids_len;
 };
 
-static void
-panic_on_fail(int error, char const *function_name)
-{
-	if (error)
-		pr_crit("%s() returned error code %d. This is too critical for a graceful recovery; I must die now.",
-		    function_name, error);
-}
-
-static void
-mutex_lock(struct thread_pool *pool)
-{
-	panic_on_fail(pthread_mutex_lock(&pool->lock), "pthread_mutex_lock");
-}
-
-static void
-mutex_unlock(struct thread_pool *pool)
-{
-	panic_on_fail(pthread_mutex_unlock(&pool->lock), "pthread_mutex_unlock");
-}
-
 /* Wait until the parent sends us work. */
 static void
 wait_for_parent_signal(struct thread_pool *pool, unsigned int thread_id)
@@ -203,7 +184,7 @@ tasks_poll(void *arg)
 	struct thread_pool_task *task;
 	unsigned int thread_id;
 
-	mutex_lock(pool);
+	mutex_lock(&pool->lock);
 
 	pool->thread_count++;
 	thread_id = pool->thread_count;
@@ -218,7 +199,7 @@ tasks_poll(void *arg)
 		/* Claim the work. */
 		task = task_queue_pull(pool, thread_id);
 		pool->working_count++;
-		mutex_unlock(pool);
+		mutex_unlock(&pool->lock);
 
 		if (task != NULL) {
 			task->cb(task->arg);
@@ -227,7 +208,7 @@ tasks_poll(void *arg)
 			task_destroy(task);
 		}
 
-		mutex_lock(pool);
+		mutex_lock(&pool->lock);
 		pool->working_count--;
 
 		if (pool->stop)
@@ -237,7 +218,7 @@ tasks_poll(void *arg)
 			signal_to_parent(pool);
 	}
 
-	mutex_unlock(pool);
+	mutex_unlock(&pool->lock);
 	pr_op_debug("Thread %s.%u: Returning.", pool->name, thread_id);
 	return NULL;
 }
@@ -248,15 +229,20 @@ thread_pool_attr_create(pthread_attr_t *attr)
 	int error;
 
 	error = pthread_attr_init(attr);
-	if (error)
-		return pr_op_errno(error, "Calling pthread_attr_init()");
+	if (error) {
+		pr_op_err_st("pthread_attr_init() returned error %d: %s",
+		    error, strerror(error));
+		return error;
+	}
 
 	/* Use 2MB (default in most 64 bits systems) */
 	error = pthread_attr_setstacksize(attr, 1024 * 1024 * 2);
 	if (error) {
 		pthread_attr_destroy(attr);
-		return pr_op_errno(error,
-		    "Calling pthread_attr_setstacksize()");
+		pr_op_err_st(
+		    "pthread_attr_setstacksize() returned error %d: %s",
+		    error, strerror(error));
+		return error;
 	}
 
 	/*
@@ -293,7 +279,8 @@ spawn_threads(struct thread_pool *pool)
 		error = pthread_create(&pool->thread_ids[i], &attr, tasks_poll,
 		    pool);
 		if (error) {
-			error = pr_op_errno(error, "Spawning pool thread");
+			pr_op_err_st("pthread_create() returned error %d: %s",
+			    error, strerror(error));
 			goto end;
 		}
 
@@ -319,23 +306,24 @@ thread_pool_create(char const *name, unsigned int threads,
 	/* Init locking */
 	error = pthread_mutex_init(&result->lock, NULL);
 	if (error) {
-		error = pr_op_errno(error, "Calling pthread_mutex_init()");
+		pr_op_err_st("pthread_mutex_init() returned error %d: %s",
+		    error, strerror(error));
 		goto free_tmp;
 	}
 
 	/* Init conditional to signal pending work */
 	error = pthread_cond_init(&result->parent2worker, NULL);
 	if (error) {
-		error = pr_op_errno(error,
-		    "Calling pthread_cond_init() at working condition");
+		pr_op_err_st("pthread_cond_init(p2w) returned error %d: %s",
+		    error, strerror(error));
 		goto free_mutex;
 	}
 
 	/* Init conditional to signal no pending work */
 	error = pthread_cond_init(&result->worker2parent, NULL);
 	if (error) {
-		error = pr_op_errno(error,
-		    "Calling pthread_cond_init() at waiting condition");
+		pr_op_err_st("pthread_cond_init(w2p) returned error %d: %s",
+		    error, strerror(error));
 		goto free_working_cond;
 	}
 
@@ -381,7 +369,7 @@ thread_pool_destroy(struct thread_pool *pool)
 	pr_op_debug("Destroying thread pool '%s'.", pool->name);
 
 	/* Remove all pending work and send the signal to stop it */
-	mutex_lock(pool);
+	mutex_lock(&pool->lock);
 	queue = &(pool->queue);
 	while (!TAILQ_EMPTY(queue)) {
 		tmp = TAILQ_FIRST(queue);
@@ -390,7 +378,7 @@ thread_pool_destroy(struct thread_pool *pool)
 	}
 	pool->stop = true;
 	pthread_cond_broadcast(&pool->parent2worker);
-	mutex_unlock(pool);
+	mutex_unlock(&pool->lock);
 
 	for (t = 0; t < pool->thread_ids_len; t++)
 		pthread_join(pool->thread_ids[t], NULL);
@@ -415,13 +403,14 @@ thread_pool_push(struct thread_pool *pool, char const *task_name,
 	struct thread_pool_task *task;
 	int error;
 
+	task = NULL;
 	error = task_create(task_name, cb, arg, &task);
 	if (error)
 		return error;
 
-	mutex_lock(pool);
+	mutex_lock(&pool->lock);
 	task_queue_push(pool, task);
-	mutex_unlock(pool);
+	mutex_unlock(&pool->lock);
 
 	/*
 	 * Note: This assumes the threads have already spawned.
@@ -437,9 +426,9 @@ thread_pool_avail_threads(struct thread_pool *pool)
 {
 	bool result;
 
-	mutex_lock(pool);
+	mutex_lock(&pool->lock);
 	result = (pool->working_count < pool->thread_ids_len);
-	mutex_unlock(pool);
+	mutex_unlock(&pool->lock);
 
 	return result;
 }
@@ -448,7 +437,7 @@ thread_pool_avail_threads(struct thread_pool *pool)
 void
 thread_pool_wait(struct thread_pool *pool)
 {
-	mutex_lock(pool);
+	mutex_lock(&pool->lock);
 
 	/* If the pool has to stop, the wait will happen during the joins. */
 	while (!pool->stop) {
@@ -467,5 +456,5 @@ thread_pool_wait(struct thread_pool *pool)
 		wait_for_worker_signal(pool);
 	}
 
-	mutex_unlock(pool);
+	mutex_unlock(&pool->lock);
 }
