@@ -1,7 +1,7 @@
 #include <check.h>
 #include "object/tal.h"
 
-#include "address.c"
+#include "types/address.c"
 
 static unsigned char db_imp_ski[] = {
     0x0e, 0xe9, 0x6a, 0x8e, 0x2f, 0xac, 0x50, 0xce, 0x6c, 0x5f,
@@ -56,7 +56,7 @@ __handle_roa_v4(uint32_t as, struct ipv4_prefix const *prefix,
 }
 
 int
-__handle_roa_v6(uint32_t as, struct ipv6_prefix const * prefix,
+__handle_roa_v6(uint32_t as, struct ipv6_prefix const *prefix,
     uint8_t max_length, void *arg)
 {
 	return rtrhandler_handle_roa_v6(arg, as, prefix, max_length);
